@@ -24,6 +24,9 @@
 #include <sys/wait.h>
 #include <stdarg.h>
 #include <string.h>
+#include <pthread.h>
+
+#include "config.h"
 
 JSObjectRef function_proto;
 
@@ -45,9 +48,9 @@ GIBaseInfo *base_info_info = 0;
 GQuark js_ref_quark;
 
 guint seed_debug_flags = 0;	/* global seed debug flag */
+gboolean seed_arg_print_version = FALSE; // Flag to print version and quit
 
-__thread JSObjectRef seed_next_gobject_wrapper = NULL;
-
+pthread_key_t seed_next_gobject_wrapper_key;
 
 #ifdef SEED_ENABLE_DEBUG
 static const GDebugKey seed_debug_keys[] = {
@@ -64,6 +67,24 @@ static const GDebugKey seed_debug_keys[] = {
 };
 #endif /* SEED_ENABLE_DEBUG */
 
+static bool 
+seed_gobject_has_instance (JSContextRef ctx, JSObjectRef constructor,
+			  JSValueRef possible_instance, JSValueRef* exception)
+{
+  GType constructor_type, value_type;
+  if (JSValueIsNull (ctx, possible_instance) ||
+      !JSValueIsObject (ctx, possible_instance) ||
+      !JSValueIsObjectOfClass (ctx, possible_instance, gobject_class))
+    return FALSE;
+  
+  constructor_type = (GType) JSObjectGetPrivate (constructor);
+  value_type = G_OBJECT_TYPE ((GObject *)
+			      JSObjectGetPrivate ((JSObjectRef) possible_instance));
+
+  return g_type_is_a (value_type, constructor_type);
+}
+
+
 /**
  * seed_prepare_global_context:
  * @ctx: A #SeedContext on which to add the default set of global objects.
@@ -166,7 +187,9 @@ seed_gobject_constructor_invoked (JSContextRef ctx,
   SEED_NOTE (INITIALIZATION, "Constructing object of type %s",
 	     g_type_name (type));
 
-  seed_next_gobject_wrapper = seed_make_wrapper_for_type (ctx, type);
+
+  pthread_setspecific (seed_next_gobject_wrapper_key, 
+		       seed_make_wrapper_for_type (ctx, type));
 
   while (i < nparams)
     {
@@ -185,7 +208,7 @@ seed_gobject_constructor_invoked (JSContextRef ctx,
 
       if (param_spec == NULL)
 	{
-	  JSObjectSetProperty (ctx, seed_next_gobject_wrapper, jsprop_name,
+	  JSObjectSetProperty (ctx, pthread_getspecific(seed_next_gobject_wrapper_key), jsprop_name,
 			       jsprop_value, 0, NULL);
 	  ++i;
 	  continue;
@@ -204,7 +227,7 @@ seed_gobject_constructor_invoked (JSContextRef ctx,
 	{
 	  g_free (params);
 	  JSPropertyNameArrayRelease (jsprops);
-	  seed_next_gobject_wrapper = NULL;
+	  pthread_setspecific(seed_next_gobject_wrapper_key, NULL);
 	  return 0;
 	}
       params[ri].name = prop_name;
@@ -456,8 +479,7 @@ seed_gobject_method_invoked (JSContextRef ctx,
       else if (dir == GI_DIRECTION_IN || dir == GI_DIRECTION_INOUT)
 	{
 
-	  if (!seed_gi_make_argument (ctx, arguments[i],
-				      type_info, arg_info,
+	  if (!seed_gi_make_argument (ctx, arguments[i], type_info,
 				      &in_args[n_in_args++], exception))
 	    {
 	      seed_make_exception (ctx, exception,
@@ -916,7 +938,8 @@ seed_gobject_set_property (JSContextRef context,
   gsize length;
   gsize i, len;
 
-  if (seed_next_gobject_wrapper || JSValueIsNull (context, value))
+  if (pthread_getspecific(seed_next_gobject_wrapper_key) ||
+      JSValueIsNull (context, value))
     return 0;
 
 
@@ -1102,7 +1125,7 @@ JSClassDefinition gobject_constructor_def = {
   NULL,				/* Get Property Names */
   NULL,				/* Call As Function */
   seed_gobject_constructor_invoked,	/* Call As Constructor */
-  NULL,				/* Has Instance */
+  seed_gobject_has_instance,				/* Has Instance */
   seed_gobject_constructor_convert_to_type
 };
 
@@ -1122,7 +1145,7 @@ JSClassDefinition gobject_named_constructor_def = {
   NULL,				/* Get Property Names */
   NULL,				/* Call As Function */
   seed_gobject_named_constructor_invoked,	/* Call As Constructor */
-  NULL,				/* Has Instance */
+  seed_gobject_has_instance,				/* Has Instance */
   seed_gobject_constructor_convert_to_type
 };
 
@@ -1246,6 +1269,8 @@ static GOptionEntry seed_args[] = {
   {"seed-no-debug", 0, 0, G_OPTION_ARG_CALLBACK, seed_arg_no_debug_cb,
    "Disable Seed debugging", "FLAGS"},
 #endif /* SEED_ENABLE_DEBUG */
+  {"seed-version", 0, 0, G_OPTION_ARG_NONE, &seed_arg_print_version,
+   "Print libseed version", 0},
   {NULL,},
 };
 
@@ -1277,7 +1302,7 @@ seed_parse_args (int *argc, char ***argv)
   /* Initiate any command line options from the backend */
 
   seed_group = seed_get_option_group ();
-  g_option_context_set_main_group (option_context, seed_group);
+  g_option_context_add_group (option_context, seed_group);
 
   if (!g_option_context_parse (option_context, argc, argv, &error))
     {
@@ -1295,6 +1320,16 @@ seed_parse_args (int *argc, char ***argv)
   return ret;
 }
 
+void
+seed_engine_destroy (SeedEngine *eng)
+{
+  JSValueUnprotect (eng->context, eng->global);
+  JSGlobalContextRelease (eng->context);
+  JSContextGroupRelease (eng->group);
+  
+  g_free (eng);
+}
+
 /**
  * seed_init_with_context_group:
  * @argc: A reference to the number of arguments remaining to parse.
@@ -1316,14 +1351,19 @@ seed_init_with_context_group (gint * argc,
 {
 
   g_type_init ();
-  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_WARNING, seed_log_handler,
-		     0);
+  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_WARNING, seed_log_handler, 0);
 
   if ((argc != 0) && seed_parse_args (argc, argv) == FALSE)
     {
       SEED_NOTE (MISC, "failed to parse arguments.");
       return FALSE;
     }
+  
+  if (seed_arg_print_version)
+    {
+      g_print("%s\n", "Seed " VERSION);
+      exit(EXIT_SUCCESS);
+    }
 
   qname = g_quark_from_static_string ("js-type");
   qprototype = g_quark_from_static_string ("js-prototype");
@@ -1408,6 +1448,7 @@ SeedEngine *
 seed_init (gint * argc, gchar *** argv)
 {
   context_group = JSContextGroupCreate ();
+  pthread_key_create(&seed_next_gobject_wrapper_key, NULL);
 
   return seed_init_with_context_group (argc, argv, context_group);
 }
