@@ -17,9 +17,6 @@
  * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
  */
 
-// TODO: Don't use GNU libc extension canonicalize_file_name, don't use realpath?
-#define _GNU_SOURCE
-
 #include <gio/gio.h>
 #include <string.h>
 
@@ -246,7 +243,8 @@ seed_gi_importer_handle_struct (JSContextRef ctx,
   GIFunctionInfo *finfo;
 
   struct_ref = JSObjectMake (ctx, seed_struct_constructor_class, info);
-
+  g_base_info_ref (info);
+  
   n_methods = g_struct_info_get_n_methods (info);
 
   for (i = 0; i < n_methods; i++)
@@ -295,6 +293,7 @@ seed_gi_importer_handle_union (JSContextRef ctx,
   GIFunctionInfo *finfo;
 
   union_ref = JSObjectMake (ctx, seed_struct_constructor_class, info);
+  g_base_info_ref (info);
 
   n_methods = g_union_info_get_n_methods (info);
 
@@ -692,7 +691,7 @@ seed_importer_handle_file (JSContextRef ctx,
 					g_path_get_dirname (file_path), NULL);
     }
 
-  normalized_path = canonicalize_file_name (absolute_path);
+  normalized_path = realpath (absolute_path, NULL);
 
   js_file_dirname = seed_value_from_string (ctx, normalized_path, NULL);
 
